import { Component, Injectable, Input, NgModule } from '@angular/core';
import { CommonModule } from '@angular/common';
import { Subject as Subject$1 } from 'rxjs/Subject';
import { share as share$1 } from 'rxjs/operator/share';

class LoadingBarService {
    constructor() {
        this.pending = new Subject$1();
        this._pendingRequests = 0;
    }
    /**
     * @param {?=} source$
     * @return {?}
     */
    startLoading(source$) {
        this.pending.next({
            started: this._pendingRequests === 0,
            pendingRequests: ++this._pendingRequests,
        });
        if (source$) {
            share$1.call(source$).subscribe(this.getLoadingObserver());
        }
    }
    /**
     * @return {?}
     */
    endLoading() {
        if (this._pendingRequests === 0) {
            return;
        }
        this.pending.next({
            completed: this._pendingRequests === 1,
            pendingRequests: --this._pendingRequests,
        });
    }
    /**
     * @return {?}
     */
    getLoadingObserver() {
        return {
            next: (x) => x,
            error: (err) => this.endLoading(),
            complete: () => this.endLoading(),
        };
    }
    /**
     * @return {?}
     */
    ngOnDestroy() {
        this.pending.complete();
    }
}
LoadingBarService.decorators = [
    { type: Injectable },
];
/**
 * @nocollapse
 */
LoadingBarService.ctorParameters = () => [];

class LoadingBarComponent {
    /**
     * @param {?} loadingBarService
     */
    constructor(loadingBarService) {
        this.includeSpinner = true;
        this.started = false;
        this.progressWidth = '0';
        this._latencyThreshold = 10;
        this._startSize = 0.02;
        this._status = 0;
        loadingBarService.pending.subscribe((progress) => {
            if (progress.started)
                this.start();
            if (progress.completed)
                this.complete();
        });
    }
    /**
     * Inserts the loading bar element into the dom, and sets it to 2%
     * @return {?}
     */
    start() {
        this._startTimeout = setTimeout(() => {
            clearTimeout(this._completeTimeout);
            // do not continually broadcast the started event:
            if (this.started) {
                return;
            }
            this.started = true;
            this._status = 0;
            this.set(this._startSize);
        }, this._latencyThreshold);
    }
    /**
     * Set the loading bar's width to a certain percent.
     *
     * @param {?} n any value between 0 and 1
     * @return {?}
     */
    set(n) {
        if (!this.started) {
            return;
        }
        const /** @type {?} */ pct = (n * 100) + '%';
        this.progressWidth = pct;
        this._status = n;
        // increment loadingbar to give the illusion that there is always
        // progress but make sure to cancel the previous timeouts so we don't
        // have multiple incs running at the same time.
        clearTimeout(this._incTimeout);
        this._incTimeout = setTimeout(() => this.inc(), 250);
    }
    /**
     * @return {?}
     */
    complete() {
        setTimeout(() => {
            this.set(1);
            clearTimeout(this._completeTimeout);
            clearTimeout(this._startTimeout);
            // Attempt to aggregate any start/complete calls within 500ms:
            this._completeTimeout = setTimeout(() => this.started = false, 500);
        }, this._latencyThreshold);
    }
    /**
     * Increments the loading bar by a random amount
     * but slows down as it progresses
     * @return {?}
     */
    inc() {
        if (this._status >= 1) {
            return;
        }
        let /** @type {?} */ rnd = 0;
        let /** @type {?} */ stat = this._status;
        if (stat >= 0 && stat < 0.25) {
            // Start out between 3 - 6% increments
            rnd = (Math.random() * (5 - 3 + 1) + 3) / 100;
        }
        else if (stat >= 0.25 && stat < 0.65) {
            // increment between 0 - 3%
            rnd = (Math.random() * 3) / 100;
        }
        else if (stat >= 0.65 && stat < 0.9) {
            // increment between 0 - 2%
            rnd = (Math.random() * 2) / 100;
        }
        else if (stat >= 0.9 && stat < 0.99) {
            // finally, increment it .5 %
            rnd = 0.005;
        }
        else {
            // after 99%, don't increment:
            rnd = 0;
        }
        this.set(this._status + rnd);
    }
}
LoadingBarComponent.decorators = [
    { type: Component, args: [{
                selector: 'ng-loading-bar, ngx-loading-bar',
                template: `
    <ng-container *ngIf="started">
        <div id="loading-bar-spinner" *ngIf="includeSpinner"><div class="spinner-icon"></div></div>
        <div id="loading-bar"><div class="bar" [style.width]="progressWidth"><div class="peg"></div></div></div>
    </ng-container>
    `,
            },] },
];
/**
 * @nocollapse
 */
LoadingBarComponent.ctorParameters = () => [
    { type: LoadingBarService, },
];
LoadingBarComponent.propDecorators = {
    'includeSpinner': [{ type: Input },],
};

class LoadingBarModule {
    /**
     * @return {?}
     */
    static forRoot() {
        return {
            ngModule: LoadingBarModule,
            providers: [LoadingBarService],
        };
    }
}
LoadingBarModule.decorators = [
    { type: NgModule, args: [{
                imports: [CommonModule],
                declarations: [LoadingBarComponent],
                exports: [LoadingBarComponent],
            },] },
];
/**
 * @nocollapse
 */
LoadingBarModule.ctorParameters = () => [];

/**
 * Generated bundle index. Do not edit.
 */

export { LoadingBarModule, LoadingBarService, LoadingBarComponent };
//# sourceMappingURL=core.js.map
